CON
  _clkmode = xtal1 + pll16x     
  _xinfreq = 5_000_000

  START_BYTE           =  $FF
  TIMEOUT_MAX          =  $0000_0FFF
  PACKET_LEN_MAX       =  32

  {Packet indexes}
  PACKET_LENGTH_IDX   = 0
  PACKET_COMMAND_IDX  = 1
  PACKET_PAYLOAD_IDX  = 2

  {Command types}
  READ_COMMAND        = 0  
  WRITE_COMMAND       = 1  
  SECRET_COMMAND      = 10 
                      

OBJ
  serial        : "FullDuplexSerial"
  
DAT
  buff          byte    $0[PACKET_LEN_MAX]
  workSpace     byte    $0[PACKET_LEN_MAX] 
  
PUB Main | unitTest
  serial.Start(31, 30, %0000, 115_200)
  waitcnt(cnt + (1 * clkfreq))

  unitTest := false

  if(unitTest)
    serial.str(string("Byte to word demo", 13, "------------------------", 13))
    FillBuffer
    PrintDebugOutput
  else
    repeat 
      GetPacket
      EchoPacket
  

PRI GetPacket | i, len, timeout
{
  Get and frame a packet.
  Block untils an $FF is received.
  Timeout if a second $FF does not come along
  Fills @buff with packet bytes
}
  timeout := TIMEOUT_MAX
   i := 0
  'repeat
    repeat until serial.rx == START_BYTE
    repeat until serial.rxcheck == START_BYTE
      if(timeout-- < 0)
        timeout := TIMEOUT_MAX
        'serial.str(string("2"))
        serial.RxFlush 
        quit

    buff[i++] := serial.rx
    repeat buff[PACKET_LENGTH_IDX]-1
      buff[i++] := serial.rx

PRI EchoPacket | i
  i := 0 
  repeat buff[PACKET_LENGTH_IDX]
    serial.tx(buff[i++])
    
PRI ValidatePacket | checksum, i
{
  Validate the check sum.
  The checksum is an XOR of all packet byte
  except the two start bytes
}
  checksum := i := 0
  repeat buff[PACKET_LENGTH_IDX]-1
    checksum ^= buff[i++]

  return checksum == buff[i]  

PRI PrintDebugOutput | t1
  serial.str(string("Filled buffer with "))
  serial.dec(buff[PACKET_LENGTH_IDX])
  serial.str(string(" bytes",13))
   
  serial.str(string("Checksum Validation = "))
  t1 := ValidatePacket
  serial.dec(t1)
  serial.tx(13)
   
  serial.str(string("Command = "))
  PrintCommandType(buff[WRITE_COMMAND])
  serial.tx(13)
   
  PrintPacketByByte
  serial.tx(13)
  PrintPacketByWord
  serial.tx(13)
  PrintPacketByLong
  serial.tx(13)

PRI FillBuffer | i
{
  Manually fill the buffer for unit testing packet processing
}
  i := 0
  buff[i++] := 7
  buff[i++] := 10
  buff[i++] := 1
  buff[i++] := 2
  buff[i++] := 3
  buff[i++] := 4
  buff[i++] := 9



{{-----------------------------------------
  Debug Helper methods
-----------------------------------------}}
PRI PrintPacketByWord | thisManyTimes, i
    thisManyTimes := buff[PACKET_LENGTH_IDX] / 2 +  buff[PACKET_LENGTH_IDX] // 2
    i := 0
    serial.str(string("Packet by WORD",13))
    repeat i from 0 to thisManyTimes - 1
      serial.hex(word[@buff][i],4)
      serial.tx(13)

PRI PrintPacketByByte | i
  i := 0

 serial.str(string("Packet by BYTE",13)) 
  repeat buff[PACKET_LENGTH_IDX]
    serial.hex(buff[i++],2)
    serial.tx(13)

PRI PrintWorkspacePacketByByte | i
  i := 0

 serial.str(string("Packets by BYTE",13)) 
  repeat buff[PACKET_LENGTH_IDX]-2
    serial.hex(workSpace[i++],2)
    serial.tx(13)


PRI PrintPacketByLong  | thisManyTimes, i, offset

  GetPacketPayload
  
  thisManyTimes := (buff[PACKET_LENGTH_IDX] -3) / 4
  if((buff[PACKET_LENGTH_IDX] - 3) // 2 > 0)
    thisManyTimes++

  i := 0
  serial.str(string("Packet payload by LONG",13))
  repeat i from 0 to thisManyTimes - 1
    serial.hex(long[@workSpace][i],8)
    serial.tx(13)

PRI PrintCommandType(cmd)
  case cmd
    0: serial.str(string("Read", 13))
    1: serial.str(string("Write", 13))
    10: serial.str(string("Secret", 13))  

PRI EchoForever
{ Used to test the PC receive logic}
  repeat
    serial.tx(serial.rx)
    
{{-----------------------------------------
  Convert to LONG helper
-----------------------------------------}}
PRI GetPacketPayload | i, j
{
  Moves the packet payload to @workspace and
  trims the first two byte, Length and Command, and the last byte, Checksum.
  This makes it a little easier to conver the payload bytes to WORD aligned
}
  bytemove(@workSpace, @buff[PACKET_PAYLOAD_IDX], buff[PACKET_LENGTH_IDX]-PACKET_PAYLOAD_IDX-1)